#!/usr/bin/perl 
use warnings;
use strict;
use Getopt::Long;


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Richard Emes University of Nottingham 2015

my $usage = "===============================================================================================================
=============================================================================================================== 
USAGE:

-i	input fasta file to process
-s	stop codon treatment one of 'norm' 'amber' 'q' where norm = codons TAA, TAG, TGA translated to '*', amber = TAG stop translated as Q, q = codons TAA, TAG, TGA translated to 'Q'
-f	frame 1,2,3 = frame +1,+2,+3


example use -f input.fa -s amber -f 1
===============================================================================================================
";


my ($input, $stop, $frame);

GetOptions(
    'i|input:s'     => \$input, 
	's|stop:s'     => \$stop,
	'f|frame:s'     => \$frame,
	);


	
if( ! defined $input) {
print "$usage\nWARNING: Cannot proceed without input fasta file\n\n"; exit;
}
if( ! defined $stop) {
print "$usage\nWARNING: Cannot proceed without stop definition\n\n"; exit;
}
if( ! defined $frame) {
print "$usage\nWARNING: Cannot proceed without frame to translate\n\n"; exit;
}


my %CODON_TABLE;

if ($stop eq "norm")
	{
%CODON_TABLE = (
   TCA => 'S',TCG => 'S',TCC => 'S',TCT => 'S',
   TTT => 'F',TTC => 'F',TTA => 'L',TTG => 'L',
   TAT => 'Y',TAC => 'Y',TAA => '*',TAG => '*',
   TGT => 'C',TGC => 'C',TGA => '*',TGG => 'W',	   
   CTA => 'L',CTG => 'L',CTC => 'L',CTT => 'L',
   CCA => 'P',CCG => 'P',CCC => 'P',CCT => 'P',
   CAT => 'H',CAC => 'H',CAA => 'Q',CAG => 'Q',
   CGA => 'R',CGG => 'R',CGC => 'R',CGT => 'R',
   ATT => 'I',ATC => 'I',ATA => 'I',ATG => 'M',
   ACA => 'T',ACG => 'T',ACC => 'T',ACT => 'T',
   AAT => 'N',AAC => 'N',AAA => 'K',AAG => 'K',
   AGT => 'S',AGC => 'S',AGA => 'R',AGG => 'R',
   GTA => 'V',GTG => 'V',GTC => 'V',GTT => 'V',
   GCA => 'A',GCG => 'A',GCC => 'A',GCT => 'A',
   GAT => 'D',GAC => 'D',GAA => 'E',GAG => 'E',
   GGA => 'G',GGG => 'G',GGC => 'G',GGT => 'G');
}

elsif ($stop eq "q")
	{
	%CODON_TABLE = (
   TCA => 'S',TCG => 'S',TCC => 'S',TCT => 'S',
   TTT => 'F',TTC => 'F',TTA => 'L',TTG => 'L',
   TAT => 'Y',TAC => 'Y',TAA => 'Q',TAG => 'Q',
   TGT => 'C',TGC => 'C',TGA => 'Q',TGG => 'W',	   
   CTA => 'L',CTG => 'L',CTC => 'L',CTT => 'L',
   CCA => 'P',CCG => 'P',CCC => 'P',CCT => 'P',
   CAT => 'H',CAC => 'H',CAA => 'Q',CAG => 'Q',
   CGA => 'R',CGG => 'R',CGC => 'R',CGT => 'R',
   ATT => 'I',ATC => 'I',ATA => 'I',ATG => 'M',
   ACA => 'T',ACG => 'T',ACC => 'T',ACT => 'T',
   AAT => 'N',AAC => 'N',AAA => 'K',AAG => 'K',
   AGT => 'S',AGC => 'S',AGA => 'R',AGG => 'R',
   GTA => 'V',GTG => 'V',GTC => 'V',GTT => 'V',
   GCA => 'A',GCG => 'A',GCC => 'A',GCT => 'A',
   GAT => 'D',GAC => 'D',GAA => 'E',GAG => 'E',
   GGA => 'G',GGG => 'G',GGC => 'G',GGT => 'G');
	}
	
elsif ($stop eq "amber")
	{
%CODON_TABLE = (
   TCA => 'S',TCG => 'S',TCC => 'S',TCT => 'S',
   TTT => 'F',TTC => 'F',TTA => 'L',TTG => 'L',
   TAT => 'Y',TAC => 'Y',TAA => '*',TAG => 'Q',
   TGT => 'C',TGC => 'C',TGA => '*',TGG => 'W',	   
   CTA => 'L',CTG => 'L',CTC => 'L',CTT => 'L',
   CCA => 'P',CCG => 'P',CCC => 'P',CCT => 'P',
   CAT => 'H',CAC => 'H',CAA => 'Q',CAG => 'Q',
   CGA => 'R',CGG => 'R',CGC => 'R',CGT => 'R',
   ATT => 'I',ATC => 'I',ATA => 'I',ATG => 'M',
   ACA => 'T',ACG => 'T',ACC => 'T',ACT => 'T',
   AAT => 'N',AAC => 'N',AAA => 'K',AAG => 'K',
   AGT => 'S',AGC => 'S',AGA => 'R',AGG => 'R',
   GTA => 'V',GTG => 'V',GTC => 'V',GTT => 'V',
   GCA => 'A',GCG => 'A',GCC => 'A',GCT => 'A',
   GAT => 'D',GAC => 'D',GAA => 'E',GAG => 'E',
   GGA => 'G',GGG => 'G',GGC => 'G',GGT => 'G');
}





my %fasta;
my $seq_id;
my $fasta_sequence;
my @sequence;
my @ids;

open CDNA, $input;

local $/ = '>'; 
<CDNA>;                                  # throw away the first line 'cos will only contain ">"


while (<CDNA>) {
    chomp $_;
    ($seq_id, @sequence) = split "\n";            # split the fasta input into Id and sequence
    $fasta_sequence = join '',@sequence;          # reassembles the sequence
	
	if ($frame ==2)
		{
		$fasta_sequence = substr($fasta_sequence, 1);
		}
	
	if ($frame ==3)
		{
		$fasta_sequence = substr($fasta_sequence, 2);
		}
	
	my @codons = ();
	my @cdna2pep = ();
	@codons = split (/(.{3})/, $fasta_sequence);
	my $codon = "";

	foreach $codon (@codons)
	{
	chomp $codon;
	my $length = length $codon;
	if ($length == 3)
		{
		if (exists $CODON_TABLE{uc $codon}){push @cdna2pep, $CODON_TABLE{uc $codon};}
		else {push @cdna2pep, "u";}
		}
	}
    
	my $cdna2pep = join '', @cdna2pep;
	print "\>$seq_id\n$cdna2pep\n";
}
